package ru.dbotthepony.mc.otm.capability;

import net.minecraft.world.item.Item;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collection;
import java.util.UUID;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public interface IPatternStorage {

	/**
	 * @return unmodifiable collection of stored patterns
	 */
	@Nonnull
	Collection<PatternState> getStoredPatterns();

	@Nonnull
	default Collection<PatternState> findPatterns(Item item) {
		return findPatterns((item2) -> item.equals(item2.item()));
	}

	@Nonnull
	default Collection<PatternState> findPatterns(Predicate<PatternState> predicate) {
		return getStoredPatterns().stream().filter(predicate).collect(Collectors.toList());
	}

	@Nullable
	default PatternState findPattern(Item item) {
		return findPattern((item2) -> item.equals(item2.item()));
	}

	@Nullable
	default PatternState findPattern(Predicate<PatternState> predicate) {
		for (var pattern : getStoredPatterns()) {
			if (predicate.test(pattern)) {
				return pattern;
			}
		}

		return null;
	}

	@Nullable
	default PatternState getPattern(@Nullable UUID id) {
		if (id == null)
			return null;

		for (var pattern : getStoredPatterns()) {
			if (pattern.id().equals(id)) {
				return pattern;
			}
		}

		return null;
	}

	default boolean hasPattern(@Nonnull PatternState state) {
		return getPattern(state.id()) != null;
	}

	default boolean hasPattern(@Nullable UUID id) {
		return getPattern(id) != null;
	}

	int getCapacity();
	int getStored();

	/**
	 * Prefer way to call it is to first call only_update = true
	 * and if it fail, try only_update = false
	 *
	 * @param pattern pattern to be inserted or update value from
	 * @param only_update do not insert new pattern if this pattern's UUID is not matched
	 * @param simulate whenever to affect state
	 * @return record of status of the operation (at status() FAIL, UPDATED, INSERTED) as well as new_state and old_state
	 */
	PatternInsertStatus insertPattern(PatternState pattern, boolean only_update, boolean simulate);

	default PatternInsertStatus insertPattern(PatternState pattern, boolean simulate) {
		return insertPattern(pattern, false, simulate);
	}

	default PatternInsertStatus updatePattern(PatternState pattern, boolean simulate) {
		return insertPattern(pattern, true, simulate);
	}
}
